package com.ruoyi.system.service;

import java.util.List;
import com.baomidou.mybatisplus.extension.service.IService;
import com.ruoyi.system.domain.VideoCall;

/**
 * VideoCallService接口
 * 
 * @author ruoyi
 * @date 2024-03-05
 */
public interface IVideoCallService extends IService<VideoCall>
{
    /**
     * 查询VideoCall
     * 
     * @param id VideoCall主键
     * @return VideoCall
     */
    public VideoCall selectVideoCallById(Long id);

    /**
     * 查询VideoCall列表
     * 
     * @param videoCall VideoCall
     * @return VideoCall集合
     */
    public List<VideoCall> selectVideoCallList(VideoCall videoCall);

    /**
     * 新增VideoCall
     * 
     * @param videoCall VideoCall
     * @return 结果
     */
    public int insertVideoCall(VideoCall videoCall);

    /**
     * 修改VideoCall
     * 
     * @param videoCall VideoCall
     * @return 结果
     */
    public int updateVideoCall(VideoCall videoCall);

    /**
     * 批量删除VideoCall
     * 
     * @param ids 需要删除的VideoCall主键集合
     * @return 结果
     */
    public int deleteVideoCallByIds(Long[] ids);

    /**
     * 删除VideoCall信息
     * 
     * @param id VideoCall主键
     * @return 结果
     */
    public int deleteVideoCallById(Long id);
}
